/*
 * 29.08.2004 by Andreas Grunewald
 */
package org.jd3lib;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.nio.charset.Charset;

import org.jd3lib.util.DebugOut;

/**
 * <p>
 * 4&#46;2&#46; Text information frames.
 * </p>
 * 
 * <p>
 * The text information frames are the most important frames, containing
 * information like artist, album and more. There may only be one text
 * information frame of its kind in an tag. If the textstring is followed by a
 * termination ($00 (00)) all the following information should be ignored and
 * not be displayed. All text frame identifiers begin with "T". Only text frame
 * identifiers begin with "T", with the exception of the "TXXX" frame. All the
 * text information frames have the following format:
 * </p>
 * 
 * <p>
 * &lt;Header for 'Text information frame', ID: "T000" - "TZZZ", excluding
 * "TXXX" described in 4.2.2.&gt;
 * </p>
 * 
 * <p>
 * Text encoding $xx
 * </p>
 * 
 * <p>
 * Information &lt;text string according to encoding&gt;
 * </p>
 * 
 * @author Andreas Grunewald
 *  
 */
public abstract class Id3FrameTBase
    extends Id3Frame {

  /**
   * @param frameData
   * @param header
   */
  public Id3FrameTBase(ByteBuffer frameData, Id3v2FrameHeader header) {
    super(frameData, header);
  }

  /**
   * @param tagData
   */

  private String Information;

  private FieldEncoding TextEncoding;

  /**
   * @return Returns the information.
   */
  public String getInformation() {
    return Information;
  }

  /**
   * @return Returns the textEncoding.
   */
  public FieldEncoding getTextEncoding() {
    return TextEncoding;
  }

  protected void initializeExisting(ByteBuffer frameData,
      Id3v2FrameHeader header) {
    TextEncoding = new FieldEncoding(frameData.get());
    if (frameData.hasRemaining())

      Information = Charset.forName("ISO-8859-1").decode(frameData).toString();
    else
      Information = "";

  }

  protected void initializeExisting(InputStream tagData,
      Id3v2FrameHeader header2) {
    DebugOut.println("RunningTbase initialize");
    byte[] temp = new byte[header2.getSize()];
    try {
      tagData.read(temp);
    } catch (IOException e) {
      //LATER 1.5 Auto-generated catch block
      e.printStackTrace();
    }
    //FIXME 0.9 Implement Encoding
    TextEncoding = new FieldEncoding(temp[0]);
    if (temp.length > 1)
      Information = new String(temp, 1, temp.length - 1);
    else
      Information = "";

  }

  protected void initializeNew(Id3v2FrameHeader header) {
    TextEncoding = new FieldEncoding(FieldEncoding.ISO88591);
    Information = "";
    header.setSize(1);
  }

  /**
   * @param information
   *          The information to set .
   */
  public void setInformation(String information) {
    Information = information;
  }

  /**
   * @param textEncoding
   *          The textEncoding to set.
   */
  public void setTextEncoding(FieldEncoding textEncoding) {
    TextEncoding = textEncoding;
  }

  /*
   * (non-Javadoc)
   * 
   * @see de.gruni.thejee.Id3Frame#getDataString()
   */
  protected String getFrameString() {
    return "Encoding: " + TextEncoding + "\nInformation: " + Information;
  }

  protected ByteArrayOutputStream getFrameData() {
    ByteArrayOutputStream out = new ByteArrayOutputStream();
    try {
      out.write(TextEncoding.getBytes());
      out.write(Information.getBytes());
    } catch (IOException e) {
      e.printStackTrace();
    }
    return out;
  }
}