/*
 * 12.10.2004
 * by Andreas Grunewald
 */
package org.jd3lib;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;

/**
 * <table WIDTH="90%" BGCOLOR="#FFFFFF">
 * <tr>
 * <td width="100%" height="38" bgcolor="#BBBBBB"><font SIZE="+2"
 * color="#FFFFFF">MPEG Audio Layer I/II/III frame header </font></td>
 * </tr>
 * <tr>
 * <td width="100%"  valign="top" background="images/code.gif"><dir><font
 * face="Arial, Helvetica, sans-serif">
 * <p>
 * &nbsp;
 * </p>
 * <p>
 * There is no main file header in an MPEG audio file. An MPEG audio file is
 * built up from a succession of smaller parts called frames. A frame is a
 * datablock with its own header and audio information.
 * </p>
 * &nbsp;&nbsp;&nbsp;&nbsp;In the case of Layer I or Layer II, frames are some
 * totally independent items, so you can cut any part of MPEG file and play it
 * correctly. The player will then play the music starting to the first plain
 * valid frame founded. However, in the case of Layer III, frames are not always
 * independant. Due to the possible use of the &quot;byte reservoir&quot;, wich
 * is a kind of buffer, frames are often dependent of each other. In the worst
 * case, 9 frames may be needed before beeing able to decode one frame.
 * <p>
 * &nbsp;&nbsp;&nbsp;&nbsp;When you want to read info about an MPEG audio file,
 * it is usually enough to find the first frame, read its header and assume that
 * the other frames are the same. But this is not always the case, as variable
 * bitrate (VBR) files may be encountered. In a VBR file, the bitrate can be
 * changed in each frame. It can be used, as an exemple to keep a constant sound
 * quality during the whole file, by using more bits where the music need more
 * to be encoded.
 * <p>
 * &nbsp;&nbsp;&nbsp;&nbsp;The frame header is 32 bits (4 bytes) length. The
 * first twelve bits (or first eleven bits in the case of the MPEG 2.5
 * extension) of a frame header are always set to 1 and are called &quot;frame
 * sync&quot;.
 * <p>
 * &nbsp;&nbsp;&nbsp;&nbsp;Frames may have an optional CRC checksum. It is 16
 * bits long and, if it exists, follows the frame header. After the CRC comes
 * the audio data. By re-calculating the CRC and comparing its value to the
 * sored one, you can check if the frame has been altered during transmission of
 * the bitstream.
 * <p>
 * &nbsp;&nbsp;&nbsp;&nbsp;Here is a presentation of the frame header content.
 * Characters A to M are used to indicate different fields. In the table below,
 * you can see details about the content of each field.
 * <p align="center">
 * <b><font size="+2" face="Courier New, Courier, mono"> AAAAAAAA AAABBCCD
 * EEEEFFGH IIJJKLMM </font> </b>
 * <p>
 * <table border="0" cellspacing=7>
 * <tr valign=TOP bgcolor="#BBBBBB">
 * <td><b><font color="#FFFFFF" face="Arial, Helvetica, sans-serif">Sign
 * </font> </b></td>
 * <td><b><font color="#FFFFFF" face="Arial, Helvetica, sans-serif">Length
 * <br>
 * (bits) </font> </b></td>
 * <td><b><font color="#FFFFFF" face="Arial, Helvetica, sans-serif">Position
 * <br>
 * (bits) </font> </b></td>
 * <td><b><font color="#FFFFFF" face="Arial, Helvetica,
 * sans-serif">Description </font> </b></td>
 * </tr>
 * <tr valign=TOP>
 * <td bgcolor="#EEEEEE">A</td>
 * <td>11</td>
 * <td>(31-21)</td>
 * <td>Frame sync (all bits must be set)</td>
 * </tr>
 * <tr valign=TOP>
 * <td bgcolor="#EEEEEE">B</td>
 * <td>2</td>
 * <td>(20,19)</td>
 * <td>MPEG Audio version ID <br>
 * 00 - MPEG Version 2.5 (later extension of MPEG&nbsp;2) <br>
 * 01 - reserved <br>
 * 10 - MPEG Version 2 (ISO/IEC 13818-3) <br>
 * 11 - MPEG Version 1 (ISO/IEC 11172-3)
 * <p>
 * Note: MPEG Version 2.5 was added lately to the MPEG 2 standard. It is an
 * extension used for very low bitrate files, allowing the use of lower sampling
 * frequencies. If your decoder does not support this extension, it is
 * recommended for you to use 12 bits for synchronization instead of 11 bits.
 * </td>
 * </tr>
 * <tr valign=TOP>
 * <td bgcolor="#EEEEEE">C</td>
 * <td>2</td>
 * <td>(18,17)</td>
 * <td>Layer description <br>
 * 00 - reserved <br>
 * 01 - Layer III <br>
 * 10 - Layer II <br>
 * 11 - Layer I</td>
 * </tr>
 * <tr valign=TOP>
 * <td bgcolor="#EEEEEE">D</td>
 * <td>1</td>
 * <td>(16)</td>
 * <td>Protection bit <br>
 * 0 - Protected by CRC (16bit CRC follows header) <br>
 * 1 - Not protected</td>
 * </tr>
 * <tr valign=TOP>
 * <td bgcolor="#EEEEEE">E</td>
 * <td>4</td>
 * <td>(15,12)</td>
 * <td>Bitrate index <br>
 * <table border="1" cellspacing="0" cellpading="0">
 * <tr>
 * <td>bits</td>
 * <td>V1,L1</td>
 * <td>V1,L2</td>
 * <td>V1,L3</td>
 * <td>V2,L1</td>
 * <td>V2, L2 &amp; L3</td>
 * </tr>
 * <tr>
 * <td>0000</td>
 * <td>free</td>
 * <td>free</td>
 * <td>free</td>
 * <td>free</td>
 * <td>free</td>
 * </tr>
 * <tr>
 * <td>0001</td>
 * <td>32</td>
 * <td>32</td>
 * <td>32</td>
 * <td>32</td>
 * <td>8</td>
 * </tr>
 * <tr>
 * <td>0010</td>
 * <td>64</td>
 * <td>48</td>
 * <td>40</td>
 * <td>48</td>
 * <td>16</td>
 * </tr>
 * <tr>
 * <td>0011</td>
 * <td>96</td>
 * <td>56</td>
 * <td>48</td>
 * <td>56</td>
 * <td>24</td>
 * </tr>
 * <tr>
 * <td>0100</td>
 * <td>128</td>
 * <td>64</td>
 * <td>56</td>
 * <td>64</td>
 * <td>32</td>
 * </tr>
 * <tr>
 * <td>0101</td>
 * <td>160</td>
 * <td>80</td>
 * <td>64</td>
 * <td>80</td>
 * <td>40</td>
 * </tr>
 * <tr>
 * <td>0110</td>
 * <td>192</td>
 * <td>96</td>
 * <td>80</td>
 * <td>96</td>
 * <td>48</td>
 * </tr>
 * <tr>
 * <td>0111</td>
 * <td>224</td>
 * <td>112</td>
 * <td>96</td>
 * <td>112</td>
 * <td>56</td>
 * </tr>
 * <tr>
 * <td>1000</td>
 * <td>256</td>
 * <td>128</td>
 * <td>112</td>
 * <td>128</td>
 * <td>64</td>
 * </tr>
 * <tr>
 * <td>1001</td>
 * <td>288</td>
 * <td>160</td>
 * <td>128</td>
 * <td>144</td>
 * <td>80</td>
 * </tr>
 * <tr>
 * <td>1010</td>
 * <td>320</td>
 * <td>192</td>
 * <td>160</td>
 * <td>160</td>
 * <td>96</td>
 * </tr>
 * <tr>
 * <td>1011</td>
 * <td>352</td>
 * <td>224</td>
 * <td>192</td>
 * <td>176</td>
 * <td>112</td>
 * </tr>
 * <tr>
 * <td>1100</td>
 * <td>384</td>
 * <td>256</td>
 * <td>224</td>
 * <td>192</td>
 * <td>128</td>
 * </tr>
 * <tr>
 * <td>1101</td>
 * <td>416</td>
 * <td>320</td>
 * <td>256</td>
 * <td>224</td>
 * <td>144</td>
 * </tr>
 * <tr>
 * <td>1110</td>
 * <td>448</td>
 * <td>384</td>
 * <td>320</td>
 * <td>256</td>
 * <td>160</td>
 * </tr>
 * <tr>
 * <td>1111</td>
 * <td>bad</td>
 * <td>bad</td>
 * <td>bad</td>
 * <td>bad</td>
 * <td>bad</td>
 * </tr>
 * </table>
 * <p>
 * NOTES: All values are in kbps <br>
 * V1 - MPEG Version 1 <br>
 * V2 - MPEG Version 2 and Version 2.5 <br>
 * L1 - Layer I <br>
 * L2 - Layer II <br>
 * L3 - Layer III <br>
 * <br>
 * &quot;free&quot; means free format. The free bitrate must remain constant, an
 * must be lower than the maximum allowed bitrate. Decoders are not required to
 * support decoding of free bitrate streams. <br>
 * &quot;bad&quot; means that the value is unallowed.
 * <p>
 * MPEG files may feature variable bitrate (VBR). Each frame may then be created
 * with a different bitrate. It may be used in all layers. Layer III decoders
 * must support this method. Layer I & II decoders may support it.
 * <p>
 * For Layer II there are some combinations of bitrate and mode which are not
 * allowed. Here is a list of allowed combinations. <table border="1"
 * cellspacing="0" align="center">
 * <tr>
 * <td >bitrate</td>
 * <td ><div align="center">single channel </div></td>
 * <td ><div align="center">stereo </div></td>
 * <td ><div align="center">intensity stereo </div></td>
 * <td><div align="center">dual channel </div></td>
 * </tr>
 * <tr>
 * <td>free</td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td  bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * </tr>
 * <tr>
 * <td>32</td>
 * <td  bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td  bgcolor="#FFCCCC"><div align="center">no </div></td>
 * <td bgcolor="#FFCCCC"><div align="center">no </div></td>
 * <td bgcolor="#FFCCCC"><div align="center">no </div></td>
 * </tr>
 * <tr>
 * <td>48</td>
 * <td  bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td bgcolor="#FFCCCC"><div align="center">no </div></td>
 * <td bgcolor="#FFCCCC"><div align="center">no </div></td>
 * <td  bgcolor="#FFCCCC"><div align="center">no </div></td>
 * </tr>
 * <tr>
 * <td>56</td>
 * <td  bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td  bgcolor="#FFCCCC"><div align="center">no </div></td>
 * <td  bgcolor="#FFCCCC"><div align="center">no </div></td>
 * <td  bgcolor="#FFCCCC"><div align="center">no </div></td>
 * </tr>
 * <tr>
 * <td>64</td>
 * <td bgcolor="#CCFFCC" ><div align="center">yes </div></td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * </tr>
 * <tr>
 * <td>80</td>
 * <td  bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td  bgcolor="#FFCCCC"><div align="center">no </div></td>
 * <td  bgcolor="#FFCCCC"><div align="center">no </div></td>
 * <td  bgcolor="#FFCCCC"><div align="center">no </div></td>
 * </tr>
 * <tr>
 * <td>96</td>
 * <td bgcolor="#CCFFCC" ><div align="center">yes </div></td>
 * <td  bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * </tr>
 * <tr>
 * <td>112</td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td  bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td  bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * </tr>
 * <tr>
 * <td>128</td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td bgcolor="#CCFFCC" ><div align="center">yes </div></td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * </tr>
 * <tr>
 * <td>160</td>
 * <td  bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * </tr>
 * <tr>
 * <td>192</td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * </tr>
 * <tr>
 * <td>224</td>
 * <td  bgcolor="#FFCCCC"><div align="center">no </div></td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * </tr>
 * <tr>
 * <td>256</td>
 * <td bgcolor="#FFCCCC"><div align="center">no </div></td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td  bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * </tr>
 * <tr>
 * <td>320</td>
 * <td  bgcolor="#FFCCCC"><div align="center">no </div></td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * </tr>
 * <tr>
 * <td>384</td>
 * <td bgcolor="#FFCCCC"><div align="center">no </div></td>
 * <td  bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td  bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * <td  bgcolor="#CCFFCC"><div align="center">yes </div></td>
 * </tr>
 * </table></td>
 * </tr>
 * <tr valign=TOP>
 * <td bgcolor="#EEEEEE">F</td>
 * <td>2</td>
 * <td>(11,10)</td>
 * <td>Sampling rate frequency index <table border="1" cellspacing="0"
 * cellpading="0">
 * <tr>
 * <td>bits</td>
 * <td>MPEG1</td>
 * <td>MPEG2</td>
 * <td>MPEG2.5</td>
 * </tr>
 * <tr>
 * <td>00</td>
 * <td>44100&nbsp;Hz</td>
 * <td>22050&nbsp;Hz</td>
 * <td>11025&nbsp;Hz</td>
 * </tr>
 * <tr>
 * <td>01</td>
 * <td>48000&nbsp;Hz</td>
 * <td>24000&nbsp;Hz</td>
 * <td>12000&nbsp;Hz</td>
 * </tr>
 * <tr>
 * <td>10</td>
 * <td>32000&nbsp;Hz</td>
 * <td>16000&nbsp;Hz</td>
 * <td>8000&nbsp;Hz</td>
 * </tr>
 * <tr>
 * <td>11</td>
 * <td>reserv.</td>
 * <td>reserv.</td>
 * <td>reserv.</td>
 * </tr>
 * </table></td>
 * </tr>
 * <tr valign=TOP>
 * <td bgcolor="#EEEEEE">G</td>
 * <td>1</td>
 * <td>(9)</td>
 * <td>Padding bit <br>
 * 0 - frame is not padded <br>
 * 1 - frame is padded with one extra slot <br>
 * <br>
 * Padding is used to exactly fit the bitrate.As an example: 128kbps 44.1kHz
 * layer II uses a lot of 418 bytes and some of 417 bytes long frames to get the
 * exact 128k bitrate. For Layer I slot is 32 bits long, for Layer II and Layer
 * III slot is 8 bits long.</td>
 * </tr>
 * <tr valign=TOP>
 * <td bgcolor="#EEEEEE">H</td>
 * <td>1</td>
 * <td>(8)</td>
 * <td>Private bit. This one is only informative.</td>
 * </tr>
 * <tr valign=TOP>
 * <td bgcolor="#EEEEEE">I</td>
 * <td>2</td>
 * <td>(7,6)</td>
 * <td>Channel Mode <br>
 * 00 - Stereo <br>
 * 01 - Joint stereo (Stereo) <br>
 * 10 - Dual channel (2 mono channels) <br>
 * 11 - Single channel (Mono) <br>
 * <br>
 * Note: Dual channel files are made of two independant mono channel. Each one
 * uses exactly half the bitrate of the file. Most decoders output them as
 * stereo, but it might not always be the case. <br>
 * &nbsp;&nbsp;&nbsp;&nbsp;One example of use would be some speech in two
 * different languages carried in the same bitstream, and then an appropriate
 * decoder would decode only the choosen language.</td>
 * </tr>
 * <tr valign=TOP>
 * <td bgcolor="#EEEEEE">J</td>
 * <td>2</td>
 * <td>(5,4)</td>
 * <td>Mode extension (Only used in Joint stereo)
 * <p>
 * Mode extension is used to join informations that are of no use for stereo
 * effect, thus reducing needed bits. These bits are dynamically determined by
 * an encoder in Joint stereo mode, and Joint Stereo can be changed from one
 * frame to another, or even switched on or off.
 * <p>
 * Complete frequency range of MPEG file is divided in subbands There are 32
 * subbands. For Layer I & II these two bits determine frequency range (bands)
 * where intensity stereo is applied. For Layer III these two bits determine
 * which type of joint stereo is used (intensity stereo or m/s stereo).
 * Frequency range is determined within decompression algorithm. <br>
 * <br>
 * <table border="0" cellspacing="2" cellpadding="0">
 * <tr>
 * <td align=CENTER>Layer I and II</td>
 * <td align=CENTER>Layer III</td>
 * </tr>
 * <tr valign=TOP>
 * <td><table border="1" cellspacing="0" cellpading="0">
 * <tr>
 * <td>value</td>
 * <td>Layer I &amp; II</td>
 * </tr>
 * <tr>
 * <td>00</td>
 * <td>bands 4 to 31</td>
 * </tr>
 * <tr>
 * <td>01</td>
 * <td>bands 8 to 31</td>
 * </tr>
 * <tr>
 * <td>10</td>
 * <td>bands 12 to 31</td>
 * </tr>
 * <tr>
 * <td>11</td>
 * <td>bands 16 to 31</td>
 * </tr>
 * </table></td>
 * <td><table border="1" cellspacing="0" cellpading="0">
 * <tr align=CENTER>
 * <td>Intensity stereo</td>
 * <td>MS stereo</td>
 * </tr>
 * <tr align=CENTER>
 * <td>off</td>
 * <td>off</td>
 * </tr>
 * <tr align=CENTER>
 * <td>on</td>
 * <td>off</td>
 * </tr>
 * <tr align=CENTER>
 * <td>off</td>
 * <td>on</td>
 * </tr>
 * <tr align=CENTER>
 * <td>on</td>
 * <td>on</td>
 * </tr>
 * </table></td>
 * </tr>
 * </table></td>
 * </tr>
 * <tr valign=TOP>
 * <td bgcolor="#EEEEEE">K</td>
 * <td>1</td>
 * <td>(3)</td>
 * <td>Copyright <br>
 * 0 - Audio is not copyrighted <br>
 * 1 - Audio is copyrighted <br>
 * <br>
 * The copyright has the same meaning as the copyright bit on CDs and DAT tapes,
 * i.e. telling that it is illegal to copy the contents if the bit is set.</td>
 * </tr>
 * <tr valign=TOP>
 * <td bgcolor="#EEEEEE">L</td>
 * <td>1</td>
 * <td>(2)</td>
 * <td>Original <br>
 * 0 - Copy of original media <br>
 * 1 - Original media <br>
 * <br>
 * The original bit indicates, if it is set, that the frame is located on its
 * original media.</td>
 * </tr>
 * <tr valign=TOP>
 * <td bgcolor="#EEEEEE">M</td>
 * <td>2</td>
 * <td>(1,0)</td>
 * <td>Emphasis <br>
 * 00 - none <br>
 * 01 - 50/15 ms <br>
 * 10 - reserved <br>
 * 11 - CCIT J.17 <br>
 * <br>
 * The emphasis indication is here to tell the decoder that the file must be
 * de-emphasized, ie the decoder must 're-equalize' the sound after a Dolby-like
 * noise supression. It is rarely used.</td>
 * </tr>
 * </table> </font> </dir></td>
 * </tr>
 * </table> <small>&#169; 1999-2001 Gabriel Bouvigne for MP3'Tech -
 * www.mp3-tech.org </small>
 * 
 * @author Andreas Grunewald
 *  
 */
public class MPEGInfo {
    
    
    
    /**
     * B    2   (20,19)     MPEG Audio version ID*/
    public MpegAudioVersionId mpegAudioVersionId;
    
    /**
     * C    2   (18,17)     Layer description
     */
    public LayerDescription layerDescription;
    
    /**D    1   (16)    Protection bit*/
    public boolean protectionBit;

    
    
    
    /**
     * @param theStream
     */
    public MPEGInfo(FileChannel theStream) {
        ByteBuffer header = ByteBuffer.allocate(4);
        try {
            theStream.read(header);
            
        } catch (IOException e) {
            // LATER 0.95 Auto-generated catch block
            e.printStackTrace();
        }
    }

}