/*
 * $Id: Paths.java,v 1.8 2005/04/27 08:22:00 gruni Exp $
 * $Author: gruni $ 
 *
 * $Log: Paths.java,v $
 * Revision 1.8  2005/04/27 08:22:00  gruni
 * -replaced String.split with StringTokenizer because of GCJ
 * -better JavaDoc
 *
 *
 */
package org.jd3lib.archoslib;

import java.io.File;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.LinkedHashMap;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 * <p>5 - File path information for all files.</p>
 * 
 * <p>In order to minimize the library size, all paths that give the actual
 * location of files on the hard drive are stored in a special way. The path
 * names are split into the individual folder names and these are stored in the
 * string pool together with all other text strings. This way a lot of text
 * strings can be "re-used" for multiple paths.</p>
 * 
 * <pre><code>
 * struct path_str {
 *   uint_32t     length;      length of the path (number of subfolders) 
 *   string_t     folder[];    pointers to names of subfolders
 * } </code></pre>
 * 
 * <dl><dt>Example:</dt>
 * 
 * <dd>/Music/Beatles/BestOf/Help.mp3</dd>
 * 
 * <dd>&nbsp;</dd>
 * 
 * <dd>length = 3</dd>
 * 
 * <dd>folder[0] = pointer to string "Music"</dd>
 * 
 * <dd>folder[1] = pointer to string "Beatles"</dd>
 * 
 * <dd>folder[2] = pointer to string "BestOf"</dd></dl>
 * 
 * @author Grunewald
 * @version 0.9 
 */
public class Paths
    extends Section implements ArcFileProcessor {

  /**
   * TODO v1.0 write documentation for private class PathT
   * */
  private class PathT {

    private int[] theStrings;

    private final int number;

    /**
     * @param temp
     * @param counter
     * @param strings
     */
    protected PathT(String path_in, Strings stringLib, int counter) {
      number = counter;
      StringTokenizer rePath = new StringTokenizer(path_in, "\\");
      theStrings = new int[rePath.countTokens()];
      int numbas = 0;
      while (rePath.hasMoreTokens()) {
        theStrings[numbas++] = stringLib.get(rePath.nextToken());
      }
      //RepairedBUG replace all REGEX usage because GCJ has no REGEX ?
      /* String[] rePath = path_in.split("\\\\");      
       //rePath = rePath[0].split("\\Q\\\\E");
       // System.out.println(Arrays.asList(rePath));
       theStrings = new int[rePath.length];
       int numbas = 0;
       for (int i = 0; i < rePath.length; i++) {
       theStrings[numbas++] = stringLib.get(rePath[i]);
       }
       */

    }

    /**
     * @return
     */
    protected ByteBuffer toByteBuffer() {
      ByteBuffer out = ByteBuffer.allocate((theStrings.length + 1) * 4);
      out.order(ByteOrder.LITTLE_ENDIAN);
      out.putInt(theStrings.length);
      for (int i = 0; i < theStrings.length; i++) {
        out.putInt(theStrings[i]);
      }
      out.flip();
      return out;
    }

    public String toString() {
      String out = "length: " + theStrings.length + "=>";
      for (int i = 0; i < theStrings.length; i++)
        out += "[" + theStrings[i] + "]";
      return out;
    }

    /**
     * @return
     */
    protected int getOffset() {
      return number;
    }

    /**
     * @return the size in bytes which this path will take in the library
     */
    protected int getSize() {
      return 4 + theStrings.length * 4;
    }
  }

  private LinkedHashMap pathTable;

  private Vector offsets;

  // private LibraryGUI lib;

  private int count;

  private final int rootSize;

  private Strings strings;

  protected Paths(int workingDirLength, Strings strs) {
    pathTable = new LinkedHashMap();
    offsets = new Vector();
    strings = strs;
    rootSize = workingDirLength;
    count = 0;
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.archoslib.Section#toByteBuffer()
   */
  protected ByteBuffer toByteBuffer() {
    Object[] thePaths = (pathTable.values().toArray());
    ByteBuffer rest = ByteBuffer
        .allocate(((PathT) thePaths[thePaths.length - 1]).getOffset()
            + ((PathT) thePaths[thePaths.length - 1]).getSize());
    System.out.println("Allocated Paths: " + rest);
    rest.order(ByteOrder.LITTLE_ENDIAN);
    for (int i = 0; i < thePaths.length; i++) {
      // System.out.println(Arrays.toString(((PathT)thePaths[i]).get().array()));
      rest.put(((PathT) thePaths[i]).toByteBuffer());
    }
    rest.flip();
    return rest;
  }

  /**
   * @param parentFile
   * @return
   */
  protected int get(File theFile) {
    PathT current = (PathT) pathTable.get(theFile);
    if (current != null) {
      return current.getOffset();
    }
    throw new NullPointerException("This Path as not yet been added to the"
        + "Paths please make sure to initialize the Paths always "
        + "before the Files.");
  }

  /**
   * @param count2
   * @return
   */
  private int getOffset(int thePos) {
    int out = 0;
    if (thePos != 0) {
      thePos = thePos - 1; //didn't want to make thePos--
      PathT temp = (PathT) offsets.get(thePos);
      out = temp.getOffset();
      out += temp.getSize();
    }
    return out;
  }

  /**
   * @param data
   */
  public void take(ArcAudioFileDecorator data) {

    File temp = data.getTheFile().getParentFile();
    //TODO v1.0 implememt logging
    //System.out.println("Paths.put():"+temp);
    //System.out.println("Paths.put():"+temp.getPath().substring(rootSize));
    if (!(pathTable.containsKey(temp))) {
      PathT Tpa = new PathT(temp.getPath().substring(rootSize), strings,
          getOffset(count));
      pathTable.put(temp, Tpa);
      offsets.add(count, Tpa);
      count++;
    }
  }

}

